package org.jeecg.modules.api.okki.site.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.modules.api.okki.site.entity.OkkiSite;
import org.jeecg.modules.api.okki.site.entity.OkkiSiteStatusChangeLog;
import org.jeecg.modules.api.okki.site.mapper.OkkiSiteMapper;
import org.jeecg.modules.api.okki.site.param.ChangeSiteStatusParam;
import org.jeecg.modules.api.okki.site.service.IOkkiSiteService;
import org.jeecg.modules.api.okki.site.service.IOkkiSiteStatusChangeLogService;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import javax.crypto.Mac;
import javax.crypto.spec.SecretKeySpec;
import java.io.BufferedReader;
import java.io.DataOutputStream;
import java.io.InputStreamReader;
import java.net.HttpURLConnection;
import java.net.URL;
import java.nio.charset.StandardCharsets;
import java.util.HashMap;
import java.util.Map;
import java.util.TreeMap;

/**
 * @Description: 站点
 * @Author: jeecg-boot
 * @Date:   2024-04-29
 * @Version: V1.0
 */
@Slf4j
@Service
public class OkkiSiteServiceImpl extends ServiceImpl<OkkiSiteMapper, OkkiSite> implements IOkkiSiteService {

    @Value("${OKKI.BASE_URL}")
    private String URL;

    @Value("${OKKI.CLIENT_SECRET}")
    private String CLIENT_SECRET;

    @Resource
    private OkkiSiteMapper okkiSiteMapper;

    @Resource
    private IOkkiSiteStatusChangeLogService okkiSiteStatusChangeLogService;

    @Override
    public boolean updateBySiteId(OkkiSite okkiSite) {
        QueryWrapper<OkkiSite> okkiSiteQueryWrapper = new QueryWrapper<>();
        okkiSiteQueryWrapper.eq("site_id", okkiSite.getSite_id());
        okkiSiteQueryWrapper.last("limit 1");
        OkkiSite okkiSite1 = okkiSiteMapper.selectOne(okkiSiteQueryWrapper);
        okkiSite.setId(okkiSite1.getId());
        int result = okkiSiteMapper.updateById(okkiSite);
        if (result == 0) {
            return false;
        }else {
            return true;
        }
    }

    @Override
    public boolean changeStatus(OkkiSite okkiSite) {
        // 请求okki平台接口
        Map<String, String> query = new TreeMap<>();
        query.put("sign_method", "hmac-md5");
        query.put("timestamp", String.valueOf(System.currentTimeMillis()));
        query.put("site_id", String.valueOf(okkiSite.getSite_id()));
        query.put("method", "update_service_node");
        // 通过site_id获取当前站点数据status的值 +1
        QueryWrapper<OkkiSite> okkiSiteQueryWrapper = new QueryWrapper<>();
        okkiSiteQueryWrapper.eq("site_id", okkiSite.getSite_id());
        okkiSiteQueryWrapper.last("limit 1");
        OkkiSite okkiSite1 = okkiSiteMapper.selectOne(okkiSiteQueryWrapper);
        okkiSite.setId(okkiSite1.getId());
        okkiSite.setStatus(okkiSite1.getStatus()+1);
        Map<String, Object> postData = new HashMap<>();
        postData.put("status", okkiSite1.getStatus()+1);
        try {
            String queryStr = buildQueryString(query);
            String body = toJsonString(postData);
            String signStr = queryStr + body;
            query.put("signature", generateHmacMD5(signStr, CLIENT_SECRET));
            String okkiUrl = URL + "?" + buildQueryString(query);
            log.info("url:" + okkiUrl + ",body:" + body);
            String res = doPost(okkiUrl, body);
            log.info(res);
            // {"code":0,"msg":"success","now":"2024-05-06 15:34:54","data":[]}
            JSONObject jsonObject= JSONObject.parseObject(res);
            if (jsonObject.get("code").equals(0)) {
                int result = okkiSiteMapper.updateById(okkiSite);
                if (result == 0) {
                    return false;
                }else {
                    OkkiSiteStatusChangeLog okkiSiteStatusChangeLog = new OkkiSiteStatusChangeLog();
                    okkiSiteStatusChangeLog.setOkkiSiteId(okkiSite.getSite_id());
                    okkiSiteStatusChangeLog.setOldStatus(okkiSite1.getStatus());
                    okkiSiteStatusChangeLog.setNewStatus(okkiSite.getStatus());
                    okkiSiteStatusChangeLogService.save(okkiSiteStatusChangeLog);
                    return true;
                }
            }else {
                return false;
            }
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

    @Override
    public boolean okkiChangeStatus(ChangeSiteStatusParam param) {
        OkkiSite okkiSite = new OkkiSite();
        QueryWrapper<OkkiSite> okkiSiteQueryWrapper = new QueryWrapper<>();
        okkiSiteQueryWrapper.eq("site_id", param.getSite_id());
        okkiSiteQueryWrapper.last("limit 1");
        OkkiSite okkiSite1 = okkiSiteMapper.selectOne(okkiSiteQueryWrapper);
        okkiSite.setId(okkiSite1.getId());
        okkiSite.setStatus(param.getStatus());
        int result = okkiSiteMapper.updateById(okkiSite);
        if (result == 0) {
            return false;
        }else {
            OkkiSiteStatusChangeLog okkiSiteStatusChangeLog = new OkkiSiteStatusChangeLog();
            okkiSiteStatusChangeLog.setOkkiSiteId(param.getSite_id());
            okkiSiteStatusChangeLog.setOldStatus(okkiSite1.getStatus());
            okkiSiteStatusChangeLog.setNewStatus(param.getStatus());
            okkiSiteStatusChangeLogService.save(okkiSiteStatusChangeLog);
            return true;
        }
    }

    private static String buildQueryString(Map<String, String> params) {
        StringBuilder query = new StringBuilder();
        for (Map.Entry<String, String> entry : params.entrySet()) {
            if (query.length() > 0) {
                query.append("&");
            }
            query.append(entry.getKey()).append("=").append(entry.getValue());
        }
        return query.toString();
    }

    private static String generateHmacMD5(String data, String secret) throws Exception {
        SecretKeySpec keySpec = new SecretKeySpec(secret.getBytes(StandardCharsets.UTF_8), "HmacMD5");
        Mac mac = Mac.getInstance("HmacMD5");
        mac.init(keySpec);
        byte[] hmacBytes = mac.doFinal(data.getBytes(StandardCharsets.UTF_8));
        StringBuilder sb = new StringBuilder(hmacBytes.length * 2);
        for (byte b : hmacBytes) {
            sb.append(String.format("%02x", b));
        }
        return sb.toString();
    }

    private static String toJsonString(Object object) throws Exception {
        com.google.gson.Gson gson = new com.google.gson.Gson();
        return gson.toJson(object);
    }

    private static String doPost(String url, String data) throws Exception {
        java.net.URL apiUrl = new URL(url);
        System.out.println(url);
        HttpURLConnection connection = (HttpURLConnection) apiUrl.openConnection();
        connection.setRequestMethod("POST");
        connection.setRequestProperty("Content-Type", "application/json");
        connection.setDoOutput(true);
        try (DataOutputStream outputStream = new DataOutputStream(connection.getOutputStream())) {
            outputStream.write(data.getBytes(StandardCharsets.UTF_8));
            outputStream.flush();
        }

        int responseCode = connection.getResponseCode();
        if (responseCode == HttpURLConnection.HTTP_OK) {
            try (BufferedReader in = new BufferedReader(new InputStreamReader(connection.getInputStream()))) {
                String line;
                StringBuilder response = new StringBuilder();
                while ((line = in.readLine()) != null) {
                    response.append(line);
                }
                return response.toString();
            }
        }
        throw new Exception("HTTP request failed with response code: " + responseCode);
    }
}
